

function trim(str){
    str = str.replace(/^\s+/, '');
    return str.replace(/\s+$/, '');
}

/**
* 
*/
function handleOnLoad(){
    
    editor.centerOnOpener();

    editor.getButtonIds();
    
    var m; 
    
    var btnId = (m = document.location.toString().match(/\?url=(.*)$/)) ? m[1] : '';
    
    //attempt to load from the current document.
    if (btnId == "__doc__"){
        var Klib = editor.getKlib();
        var doc = window.opener.top.content.document;
        var code = Klib.innerText(doc.body);
        editor.newButton(code);
        editor.testCode(code);
    }
    else if (btnId){
        editor.loadButton(btnId);
    }
    else {
        editor.newButton();
    }
    refreshButtons();
}

/**
* 
*/
function $(id){
    return document.getElementById(id);
}

function debug(obj){
    var str = '';
    
    if (typeof(obj) == "object"){
        for(var prop in obj){
            str += prop +"=";
            try {
                str += obj[prop];
            }catch(e){
                str += "?";
            }
            str += "\n";
        }
    }
    else {
        str = obj;
    }
    alert(str);
}

var editor = {};

//flag to say if a button has been saved so we can reload custom buttons when closing the dialog
editor.reloadButtonsOnClose = false;

editor.newButton = function(code){
    //clear the code box
    $('button-code').value = code || '\
this.id = "";\n\
this.imageURL = "";\n\
this.onclick = function(){\n\
    \n\
};\n';

    $('button-code').focus();
    $('button-enabled').checked = false;
    editor.showMessage();
    refreshButtons();
}


editor.viewExamples = function(){
    opener.top._content.location='chrome://interclue/content/cluecore/buttons/examples/'; 
    close();
}

editor.loadButton = function(btnId){
    var prefId = "interclue.custombutton."+ btnId;
    var code = window.opener.top.interclue.getPref(prefId);
    if (code){
        $('button-code').value = code;
        
        var Klib = editor.getKlib();
        if (Klib){
            $('button-enabled').checked = Klib.Pref.get("CustomButton."+ btnId +".enabled", false);
        }
    }
    else {
        editor.showMessage("Unable to load button '"+ btnId +"' creating new button", "info");
        editor.newButton();
    }
}

editor.saveButton = function(){
    var code = trim($('button-code').value);
    
    //code must pass all tests before it can be saved.
    var btn = editor.testCode(code);
    if (btn){
        var prefId = "interclue.custombutton."+ btn.id;
        window.opener.top.interclue.setPref(prefId, code);
        editor.saveEnabledState();
        editor.showMessage("Button '"+ btn.id +"' saved successfully.", "success");
        editor.reloadButtonsOnClose = true;
    }
    
}


editor.generateButton = function(code){
    code = trim(code || $('button-code').value);
    
    
    //we'll have to use our main Klib library for this.
    var Klib = editor.getKlib();
    if (Klib){
        return Klib.CustomButtons.generateButton(code);
    }
    else {
        editor.showMessage("Interclue Error: unable to find main window");
        return null;
    }
}

editor.isSafeURL = function(str){
    return /^http(s)?:\/\/.*$/.test(str);
}

editor.testCode = function(origCode){
    
    var btn = editor.generateButton(origCode);

    //check for required fields.
    if (!btn){
        //chrome error
        return false;
    }
    else if (btn.err){
        editor.showMessage("Code complilation failed with error:\n"+ btn.err, "error");
        //generate button failed
        return false;
    }
    else if (!btn.id){
        editor.showMessage("this.id is required", "error");
        return false;
    }
    else if (typeof btn.id != "string" || !/^[a-zA-Z0-9\-\_]+$/.test(btn.id)){
        editor.showMessage("this.id is only allowed to contain alphanumeric characters, hyphens and underscores", "error");
        return false;
    }
    else if (btn.imageURL && !editor.isSafeURL(btn.imageURL)){
        editor.showMessage("this.imageURL must be an http or https URL.", "error");
        return false;
    }
    else if (typeof btn.onclick != "function"){
        editor.showMessage("this.onclick must be a function", "error");
        return false;
    }
    else if (btn.tooltip && typeof btn.tooltip != "string"){
        editor.showMessage("this.tooltip must be a string", "error");
        return false;
    }
    else if (btn.ondblclick && typeof btn.ondblclick != "function"){
        editor.showMessage("this.ondblclick must be a function", "error");
        return false;
    }
    else if (btn.isDisabled && typeof btn.isDisabled != "function"){
        editor.showMessage("this.isDisabled must be a function", "error");
        return false;
    }
    //passed all checks,
    return btn;
}



function refreshButtons(){
    var code = trim($('button-code').value);
    if (code == ''){
        $('button-save').setAttribute('disabled', 'true');
    }
    else {
        $('button-save').setAttribute('disabled', '');
    }
}

editor.deleteButton = function(){
 
    

    var btn = editor.generateButton();

    if (btn.id){
        var Klib = editor.getKlib();
        if (Klib){
            if (confirm("Are you sure you want to delete button '"+ btn.id +"'?")){
                Klib.CustomButtons.remove(btn.id);
                editor.newButton();
                editor.showMessage("Button "+ btn.id +" deleted successfully", "success");
                editor.reloadButtonsOnClose = true;
            }
        }
        else {
            editor.showMessage("Interclue Error: Unable to delete button.", "error");
        }
    }
    else if (btn.err){
        editor.showMessage(btn.err, "error");
    }
    else {
        editor.showMessage("Unable to delete button without an id", "error");
    }
}

editor.showMessage = function(msg, type){
    var msgbox = $('message');
    
    if (msg){
        msgbox.collapsed = false;
        type = type || "info";
        msgbox.className = "message-"+ type;
        
        var text = document.createTextNode(msg);
        if (msgbox.firstChild){
            msgbox.replaceChild(text, $('message').firstChild);
        }
        else {
            msgbox.appendChild(text);
        }
    }
    else {
        msgbox.collapsed = true;
    }
}

editor.close = function(){
    if (editor.reloadButtonsOnClose){
        var Klib = editor.getKlib();
        Klib.CustomButtons.init();
        Klib.ClueBridge.rebuildDoc();
    }
    window.close();
}


/**
* return all the current button ids.
*/
editor.getButtonIds = function(){
    var branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    var btnIds = branch.getChildList("interclue.custombutton", {});
    
    for (var i=0; i<btnIds.length; i++){
        btnIds[i] = btnIds[i].replace("interclue.custombutton.", "");
    }
    
    return btnIds;
}

editor.getKlib = function(){
    return window.opener.top.interclue.getKlib();
}

editor.saveEnabledState = function(){
    var checkbox = $('button-enabled');

    //get the current button id.
    var btn = editor.generateButton();
    
    if (!btn){
        //chrome error, message already shown, revert checkbox
        checkbox.checked = !checkbox.checked;
    }
    else if (btn.err){
        editor.showMessage("Code complilation failed with error:\n"+ btn.err, "error");
        checkbox.checked = !checkbox.checked;
    }
    else if (btn.id){
        var Klib = editor.getKlib();
        if (Klib){
            Klib.Pref.set("CustomButton."+ btn.id +".enabled", checkbox.checked);
            editor.showMessage("Options saved.", "success");
            editor.reloadButtonsOnClose = true;
        }
        else {
            editor.showMessage("Interclue Error: Unable to save option", "error");
            checkbox.checked = !checkbox.checked;
        }
    }
    else {
        editor.showMessage("this.id is required", "error");
        checkbox.checked = !checkbox.checked;
    }
}


editor.centerOnOpener = function(){    
    window.screenX = (window.opener.top.screenX + (window.opener.top.innerWidth / 2)) - ($('interclue-custom-button-editor-window').width / 2);
    window.screenY = (window.opener.top.screenY + (window.opener.top.innerHeight / 2)) - ($('interclue-custom-button-editor-window').height / 2);
}