
window.onerror = Klib.error;

var Pref = {};

Pref.prefs = {};

Pref.rebuildOnExit = false;

Pref.lastPanelShown = ''; //this needs to be saved in globals somewhere if we want to re-open the last tab.

Pref.showPanel = function(name){

    if ($('tab-box') && $('tab-box').selectTab){

        //presets tab
        if (name == 'presets' || name == 'about'){
            $('tab-box').selectTab(name);
            return;
        }
        
        if (name == 'advanced' || name == 'lastAdvanced'){
            name = Pref.lastPanelShown ? Pref.lastPanelShown : 'linkclue';
        }
        //presets panel
        else if (!name){
            name = Pref.lastPanelShown ? Pref.lastPanelShown : 'presets';
        }

        var selectbox = $('pref-list');
        selectbox.value = name.toLowerCase();        

        var id = (name.toLowerCase() + '-panel');
        
        var panels = document.getElementsByTagName('fieldset');
        
        var panelShown = false;
        
        for (var i=panels.length-1; i>=0; i--){

            var panel = panels[i];
            
            /* 
            KLUDGE : Cursor (caret) sometimes fails to appear in input text fields
            ref: https://bugzilla.mozilla.org/show_bug.cgi?id=167801 
            can't just use visibility:hidden must use display:block/none as well.
            */
            if (panel.id.toLowerCase() == id){
                panelShown = true;            
                Pref.lastPanelShown = name;
                Klib.setStyle(panel, 'display', 'block');
                Klib.setStyle(panel, 'visibility', 'visible');
            }
            else {
                Klib.setStyle(panel, 'display', 'none');
                Klib.setStyle(panel, 'visibility', 'hidden');
            }
        }
        
        if (panelShown){
            $('tab-box').selectTab('advanced');
            $('pref-list').focus();
        }
        else {
            $('tab-box').selectTab('presets');
            
            //select the first inputbox within the basic tab
            var boxes = $('presets-tab').getElementsByTagName('input');
            boxes[0].focus();
        }
    }
    else {
        //not quite ready yet.
        setTimeout(function(){
            Pref.showPanel(name);
        }, 100);
    }
}

Pref.saveForm = function(form){
    
    var eles = Pref.getEles(form);
    
    //for each preference,
    var changed = false;
    
    for (var i=0; i<eles.length; i++){
        //convert the value to the correct datatype,
        var ele = eles[i];
        var prefid = ele.getAttribute('preferenceid');
        var value = Pref.extractValue(ele);

        if (value !== null){
            value = Pref.convertToDatatype(value, ele.getAttribute('datatype'));
            
            //and overwrite the global preference
            //special case arrays?
            if (value.isArray){
                if (Pref.prefs[prefid] === undefined || (Pref.prefs[prefid].isArray && !Klib.arrayEqual(Pref.prefs[prefid], value))){
                    Pref.prefs[prefid] = value;
                    changed = true;
                    
                }
            }
            else if ((Pref.prefs[prefid] === undefined) || (Pref.prefs[prefid] != value)){
                Pref.prefs[prefid] = value;
                changed = true;
            }
        }
    }

    //hotkey must always contain a value
    //use hotkey.enable = false to disable hotkeys.
    if (!Pref.prefs['ClueFrame.hotKey'] || Pref.prefs['ClueFrame.hotKey'] == []){
        Pref.prefs['ClueFrame.hotKey'] = [Klib.KEY_CODE_CTRL];
    }
    
    //update the new password ()
    /*
    var newPassword = Klib.trim(($('User.password').value));
    if (newPassword){
        Pref.prefs['User.password'] = newPassword;
        changed = true;
    }
    */
    
    //if any have changed, save the preferences
    if (changed || Pref.rebuildOnExit){
        Pref.save();
    }

    window.close();
    
}


Pref.closeWin = function(){
    window.close();
}


/**
* returns an array of elements with preferenceid attributes from a form
*/
Pref.getEles = function(form){
    var eles = [];
    
    //get all the elements  with a preferenceId
    for (var i=0; i<form.elements.length; i++){
        if (form.elements[i].getAttribute('preferenceid')){
            eles.push(form.elements[i]);
        }
    }
    
    return eles;
}

/**
* sets the value of an input 
*/
Pref.initValue = function(ele, initValue){

    if (ele.tagName == 'INPUT' && ele.type == 'checkbox'){
        ele.checked = initValue ? true : false;
    }
    else if (ele.tagName == 'INPUT' && ele.type == 'radio'){
        ele.checked = (ele.value == initValue) ? true : false;
    }
    else if (ele.tagName == 'INPUT' && ele.type == 'text'){
        ele.value = initValue;
    }
    else if (ele.tagName == 'INPUT' && ele.type == 'hidden' && ele.getAttribute('datatype') && ele.getAttribute('datatype') == 'csv'){
        ele.value = initValue.join(',');
    }
    //don't fill in any password fields.
    else if (ele.tagName == 'INPUT' && ele.type == 'password'){
        ele.value = '';
    }
    else if (ele.tagName == 'SELECT'){
        for (var i=0; i<ele.options.length; i++){
            if (ele.options[i].value == initValue){
                ele.selectedIndex = i;
                break;
            }
        }
    }
    else if (ele.tagName == 'TEXTAREA'){
        ele.value = initValue;
    }
    else {
        Klib.error('failed to init', ele, initValue);
    }
}

/**
* initalises elements with a preferenceid attribute with the current preference value  
*/
Pref.initEles = function(doc){
    var eles = Pref.getEles($('preferences-form'), doc);

    for (var i=0; i<eles.length; i++){
        var ele = eles[i];
        var prefid = ele.getAttribute('preferenceid');
        
        if (prefid && prefid.indexOf("interclue.") == 0){
            Pref.initValue(ele, Interclue.Pref.get(prefid));
        }
        else {
            Pref.initValue(ele, Pref.prefs[prefid]);
        }
    }
    
    //what about the option buttons for the preference sets!
    Pref.initPresets();    
}


/**
* STUB: this function will be overwritten by the various platforms
*/
Pref.save = function(){
    throw Error('ERROR: Window failed to initalise properly, preferences not saved!');
}

Pref.reset = function(){
    throw Error('ERROR: Window failed to initalise properly, preferences not reset!');
}

Pref.clearAllStats = function(){
    throw Error('ERROR: Window failed to initalise properly, clearAllStats() not called!');
}

/**
* extracts a value from an input/select/optiongroup
*/
Pref.extractValue = function(ele){

    if ((ele.tagName == 'INPUT') && (ele.type == 'checkbox')){
        return (ele.checked && ele.getAttribute('value')) ? ele.getAttribute('value') : ele.checked;
    }
    else if (ele.tagName == 'INPUT' && ele.type == 'radio'){
        return ele.checked ? ele.value : null;
    }
    else {
        return ele.value;
    }
}

/**
* 
*/
Pref.resetAll = function(){
    
    if (confirm('All the options will be restored to their default values!\nAre you absolutely sure you want to do this?')){    
        var ignore = [];
    
        if (!$('reset-unsafewords').checked){
            ignore.push('Security.unsafeWords');
        }
        
        if ($('reset-statistics').checked){
            Pref.clearAllStats();
        }   
        else {
            ignore.push('cluestats');
        }
        Pref.reset(ignore.join(','));
    
        window.close();
    }
}

Pref.resetMessages = function(){
    //reset 
    Interclue.Pref.reset("hint.*");
    
    //and the temporary prefs in this dialog.
    for (var prefId in Pref.prefs){
        if (prefId.indexOf("hint.") == 0){
            Pref.prefs[prefId] = true;
        }
    }
    
    alert("Interclue Messages Reset");
}



Pref.convertToDatatype = function(value, datatype){
    datatype = datatype || 'text';
    
    switch (datatype){
        case 'email':
        case 'text':
            return  (value === false) ? '' : value;
        case 'int':
            return parseInt(value);
        case 'float':
            return parseFloat(value);
        case 'bool':
            return ((value == 'false') || !value) ? false : true;
        case 'csv':
            return Klib.csvToArray(value);
        
        default:
            throw Error('invalid datatype ['+ datatype +']');
    }
}


    



Pref.setPresets = function(ele){
    var set = ele.getAttribute("preferenceset");
    
    if (set){
        //split into key/value pairs
        var pairs = Klib.extractKeyValuePairs(set, ",", true);

        //update each pair of preferences
        for (var prefid in pairs){

            //update the editable values (if they exist)
            var editableEle = $(prefid);
            if (editableEle){
                Pref.initValue(editableEle, pairs[prefid]);
                
                //and the ones that are displayed here.
                if (editableEle.sync){
                    editableEle.sync();
                }
            }
        }
    }
}


Pref.initPresets = function(){

    var groups = ["preset-speed", "preset-performance", "preset-bandwidth"];

    for (var i=0; i<groups.length; i++){
        var group = groups[i];
        var eles = document.getElementsByName(group);
        var groupChecked = false;

        for (var j=0; j<eles.length; j++){
            var ele = eles[j];
            
            var set = ele.getAttribute("preferenceset");

            if (set){
            
                var eleChecked = true;
            
                //split into key/value pairs
                var pairs = Klib.extractKeyValuePairs(set, ",", true);

                //update each pair of preferences
                for (var prefid in pairs){
         
                    //update the editable values (if they exist)
                    var editableEle = $(prefid);
                    if (editableEle){
                        var eleVal = Pref.extractValue(editableEle);
                        
                        if (eleVal != pairs[prefid]){
                            //debug(eleVal, pairs[prefid], eleVal == pairs[prefid]);
                            eleChecked = false;
                        }
                    }
                }
                
                if (eleChecked){
                    groupChecked = true;
                }
                ele.checked = eleChecked;
            }
            else {
                //debug("set group "+ group +" = "+ groupChecked);
                ele.checked = !groupChecked;
            }
        }
    }
}


Pref.showSecretPanels = function(evt){
    if (evt.shiftKey && evt.ctrlKey){
        Pref.showPanel('beta');
    }
}


Pref.deleteCustomButton = function(btnId){
    if (confirm(_("custombuttons.question.delete", {"btnId" : btnId}))){
        Pref.rebuildOnExit = true;
        Interclue.CustomButtons.remove(btnId);
        $('interclue-preferencesdialog-custombuttons', document).innerHTML = Interclue.CustomButtons.renderButtonsTable();
        Interclue.CustomButtons.initPrefsDoc(document);
    }
}


var Sync = {};

Sync.initDoc = function(doc){
    doc = doc || document;
    
    var inputs = doc.getElementsByTagName('input');
    
    for (var i=0; i<inputs.length; i++){
        var input = inputs[i];
        if (input.getAttribute("syncTarget")){
            Sync.initEle(input);
        }
    }
}

Sync.initEle = function(ele){

    var syncId = ele.getAttribute("syncTarget");
    if (syncId){
        var target = ele.ownerDocument.getElementById(syncId);
        
        if (target){
            //all good, setup element
            ele.value = target.value;
            ele.checked = target.checked;
            
            ele.sync = function(){
                Sync.syncronize(ele);
            }
            
            ele.sync.target = target;
            ele.sync.syncing = false;
            Klib.addEvent(ele, "change", Sync.syncronize);

            target.sync = function(){
                Sync.syncronize(target);
            }
            target.sync.target = ele;
            target.sync.syncing = false;
            Klib.addEvent(target, "change", Sync.syncronize);
        }
        else {
            Klib.error("Sync: target ["+ syncId +"] not found");
        }
    }
    else {
        Klib.error("Sync: ele has no synctarget ["+ syncId +"] not found");
    }
}

Sync.syncronize = function(evt){

    var ele = evt.tagName ? evt : evt.source;

    if (ele && ele.sync && !ele.sync.syncing){
        ele.sync.syncing = true;
        ele.sync.target.sync.syncing = true;
    
        ele.sync.target.value = ele.value;
        ele.sync.target.checked = ele.checked;
        
        ele.sync.target.sync.syncing = false;
        ele.sync.syncing = false;
    }
}




var KeyCodeBox = {};
KeyCodeBox.initDoc = function(doc){
    doc = doc || document;
    
    var inputs = doc.getElementsByTagName('input');
    for (var i=0; i<inputs.length; i++){
        var ele = inputs[i];
        if (ele.getAttribute("keyCodeTarget")){
            KeyCodeBox.initEle(ele);
        }
    }
}

KeyCodeBox.initEle = function(ele){
    
    
    var targetId = ele.getAttribute("keyCodeTarget");
    if (targetId){
        var targetEle = ele.ownerDocument.getElementById(targetId);

        if (targetEle){
            //capture keypress event to show the current key combination
            Klib.addEvent(ele, "keydown", function(evt){
                if ((evt.keyCode >= 16 && evt.keyCode <= 18) || (evt.keyCode >= 65 && evt.keyCode <= 90) || (evt.keyCode >= 37 && evt.keyCode <= 40)){
                    var codes = Klib.getEventKeys(evt);
                    targetEle.value = codes.join(',');
                    ele.value = Klib.keyCodesToString(codes);
                    evt.halt();
                    return false;
                }
                //normal keypress
                else {
                    return true;
                }
            });
            
            
            if (targetEle.value){
                ele.value = Klib.keyCodesToString(Klib.csvToArray(targetEle.value));
            }
        }
        else {
            Klib.error("KeyCodeBox: Cannot find keyCodeTarget ["+ targetId +"]");
        }
    }
    else {
        Klib.error("KeyCodeBox: 'keyCodeTarget' attribute missing from ele", ele);
    }
}


//~ function showInstallDialog(){
    //~ opener.top.interclue.getKlib().Update.openInstallDialog();
//~ }


