/**
* Html tabbox
*/


var Tabbox = {};

/**
* 
*/
Tabbox.initTabbox = function(tabbox){
    //find all the tabs and tabpanels within the box
    tabbox.tabs = [];
    tabbox.tabpanels = [];
    
    tabbox.selectTab = function(tabId){

        var tab;
        
        //select by number
        if (typeof tabId == 'number'){
            tab = tabbox.tabs[tabId];
        }
        //allow users to pass a tabid
        else if (typeof tabId == "string"){
            for (var i=0; i<tabbox.tabs.length; i++){
                if (tabbox.tabs[i].tabId == tabId){
                    tab = tabbox.tabs[i];
                    break;
                }
            }
        }
        //or event
        else {
            tab = tabId.srcElement || tabId.target;

            //find the tab that was clicked
            while (tab && !tab.isTab){
                tab = tab.parentNode;
            }
        }
        
        if (tab){
            
            for (var i=0; i<tabbox.tabs.length; i++){        
                if (tabbox.tabs[i] === tab){
                    Tabbox.addClass(tab, 'tab-selected');
                    tabbox.tabpanels[i].style.visibility = 'visible';
                    tabbox.tabpanels[i].style.display = 'block';
                }
                else if (tabbox.tabs[i]){
                    Tabbox.removeClass(tabbox.tabs[i], 'tab-selected');
                    tabbox.tabpanels[i].style.visibility = 'hidden';
                    tabbox.tabpanels[i].style.display = 'none';
                }
            }
            
            tabbox.tabcurrent = tab;
            return true;
        }
        else {
            return false;
        }
    }
    
    
    tabbox.addTab = function(ele){
        
        var li = document.createElement('li');
        li.tabId = ele.getAttribute('tabid') || '';

        
        li.className = 'tab';
        li.isTab = true;
        li.tabIndex = tabbox.tabs.length;
        
        
        if (ele.getAttribute('onclick')){
            li.setAttribute('onclick', ele.getAttribute('onclick'));
        }
        
        Tabbox.addEvent(li, 'click', tabbox.selectTab);
                
        
        var span = document.createElement('span');
        span.innerHTML = ele.innerHTML;
        li.appendChild(span);
        
        tabbox.tabs.push(li);
        tabbox.tabbar.appendChild(li);
        
    }
    
    //create a tabbar
    var tabbarbox = document.createElement('div');
    tabbarbox.className = 'tab-bar-box';
    
    var tabbar = document.createElement('ul');
    tabbar.style.position = 'absolute';
    tabbar.style.top = '0px';
    tabbar.style.left = '0px';
    tabbar.style.width = Klib.isIE ? '97%' : '100%';
    tabbar.className = 'tab-bar';
    
    tabbox.tabbar = tabbar;
    tabbarbox.appendChild(tabbar);
    tabbox.insertBefore(tabbarbox, tabbox.firstChild);
    
    var panel_height = 0;
    var tab_height = 0;
    var firstTab = false;
    
    for (var i=0; i<tabbox.childNodes.length; i++){
        var ele = tabbox.childNodes[i];
        
        if (!ele.className){
            continue;
        }
        else if (Tabbox.hasClass(ele, 'tab-title') && !ele.getAttribute('disabled')){
            //build a tab from this element,
            tabbox.addTab(ele);
            
            //hide this element
            ele.style.display = 'none';

        }
        else if (Tabbox.hasClass(ele, 'tab-panel') && !ele.getAttribute('disabled')){
        
            ele.tabIndex = tabbox.tabpanels.length;
            tabbox.tabpanels[ele.tabIndex] = ele;
            
            //calculate the height of the largest panel
            // (for use when creating the box to hold it all)
            panel_height = (ele.offsetHeight > panel_height) ? ele.offsetHeight : panel_height;
            
            //hide all the tabpanels except the first (selected) one.
            ele.style.visibility = 'hidden';
        }
    }
    
    if (tabbox.tabs.length != tabbox.tabpanels.length){
        throw Error('Tabbox: Number of tabs doesn\'t match number of tabpanels');
    }
    
    //only apply if the tabbox height has not yet been specified.    
    //if (!tabbox.style.height || tabbox.style.height == 'auto'){
        //tabbox.style.height = (tab_height + panel_height) +'px';   
    //} 
    
    //select the first tab
    tabbox.selectTab(0);
}



/**
* helper functions
*/


Tabbox.getElementsByClass = function(classname){
    var eles = document.getElementsByTagName('*');
    var found = [];
    
    for (var i=0; i<eles.length; i++){
        var ele = eles[i];
        if (Tabbox.hasClass(ele, classname)){
            found.push(ele);
        }
    }
    
    return found;
}

/**
* 
*/
Tabbox.addEvent = function(obj, eventname, func){
    if (obj.attachEvent){
        obj.attachEvent('on'+ eventname, func);
    }
    else if (obj.addEventListener){
        obj.addEventListener(eventname, func, true);
    }
}


/*
* 
*/
Tabbox.addClass = function (ele, className){
    //adds a className to an element but only if that name doesn't already exist
    if (!ele.className){return}
    
    var regex = new RegExp('\\b'+ className +'\\b', "ig");
    //var exp = eval('/\\b'+ className +'\\b/ig');
    
    if (!ele.className.match(regex)){
        ele.className = (ele.className +' '+ className);
    }
}

/*
* 
*/
Tabbox.removeClass = function(ele, classes){
    if (!ele.className){return}
    
    var regex = new RegExp('\\b'+ classes +'\\b', "ig");
    var new_class = ele.className.replace(regex, '');
    
    if (new_class != ele.className){
        ele.className = new_class;
    }
}

Tabbox.hasClass = function(ele, classname){
    if (!ele.className){return false}
    var regex = new RegExp('\\b'+ classname +'\\b', "i");
    //var exp = eval('/\\b'+ classname +'\\b/i');
    return regex.test(ele.className);
}


Tabbox.initDoc = function(doc){
    
    doc = doc || document;
    
    //find the tabboxes on the page
    var tabboxes = Tabbox.getElementsByClass('tab-box');
    
    for (var i=0; i<tabboxes.length; i++){
        Tabbox.initTabbox(tabboxes[i]);
    }
}